@@ -66,6 +66,8 @@
 #include "ascii-ctype.h"
 #include "ascii-string.h"
 #include "exit-with-parent.h"
+#include "get_current_dir_name.h"
+#include "getline.h"
 #include "nbd-protocol.h"
 #include "nbdkit-string.h"
 #include "open_memstream.h"
@@ -89,9 +91,14 @@ static struct backend *open_plugin_so (size_t i, const char *filename,
 static struct backend *open_filter_so (struct backend *next, size_t i,
                                        const char *filename, int short_name);
 static void parse_key_values (int argc, char **argv, bool is_first,
-                              const char *magic_config_key);
+                              const char *magic_config_key,
+                              const char *cwd);
 static void parse_key_value (const char *arg,
-                             bool is_first, const char *magic_config_key);
+                             bool is_first, const char *magic_config_key,
+                             const char *cwd);
+static void parse_at_path (const char *filename,
+                           bool is_first, const char *magic_config_key,
+                           const char *cwd);
 static void start_serving (void);
 static void write_pidfile (void);
 static bool is_config_key (const char *key, size_t len);
@@ -110,6 +117,7 @@ enum log_to log_to = LOG_TO_DEFAULT; /* --log */
 const char *log_to_file;        /* --log=/path */
 FILE *log_to_fp;                /* --log=/path */
 unsigned mask_handshake = ~0U;  /* --mask-handshake */
+const char *process_name;       /* --name */
 bool newstyle = true;           /* false = -o, true = -n */
 bool no_mc;                     /* --no-meta-contexts */
 bool no_sr;                     /* --no-sr */
@@ -396,11 +404,8 @@ main (int argc, char *argv[])
       break;
 
     case LONG_OPTIONS_OPTION:
-      for (i = 0; long_options[i].name != NULL; ++i) {
-        if (strcmp (long_options[i].name, "long-options") != 0 &&
-            strcmp (long_options[i].name, "short-options") != 0)
-          printf ("--%s\n", long_options[i].name);
-      }
+      for (i = 0; long_options[i].name != NULL; ++i)
+        printf ("--%s\n", long_options[i].name);
       cleanup_random_fifo ();
       exit (EXIT_SUCCESS);
 
@@ -410,6 +415,10 @@ main (int argc, char *argv[])
         exit (EXIT_FAILURE);
       break;
 
+    case NAME_OPTION:
+      process_name = optarg;
+      break;
+
     case NO_MC_OPTION:
       no_mc = true;
       break;
@@ -663,7 +672,7 @@ main (int argc, char *argv[])
     /* If we will or might use syslog. */
   case LOG_TO_DEFAULT:
   case LOG_TO_SYSLOG:
-    openlog (program_name, LOG_PID, 0);
+    openlog (process_name ? : program_name, LOG_PID, 0);
     break;
 
   case LOG_TO_FILE:
@@ -872,7 +881,14 @@ main (int argc, char *argv[])
   }
 
   /* Parse key=value on the command line. */
-  parse_key_values (argc, argv, true, top->magic_config_key (top));
+  {
+    CLEANUP_FREE char *cwd = get_current_dir_name ();
+    if (cwd == NULL) {
+      fprintf (stderr, "%s: get_current_dir_name: %m\n", program_name);
+      exit (EXIT_FAILURE);
+    }
+    parse_key_values (argc, argv, true, top->magic_config_key (top), cwd);
+  }
 
   /* This must run after parsing the parameters so that the script can
    * be loaded for scripting languages.  But it must be called before
@@ -1152,21 +1168,22 @@ open_filter_so (struct backend *next, size_t i,
 /* Call config() on each of the key=value plugin & filter parameters. */
 static void
 parse_key_values (int argc, char **argv, bool is_first,
-                  const char *magic_config_key)
+                  const char *magic_config_key,
+                  const char *cwd)
 {
   if (optind >= argc) return;
 
-  parse_key_value (argv[optind], is_first, magic_config_key);
+  parse_key_value (argv[optind], is_first, magic_config_key, cwd);
 
   ++optind;
 #if defined(__GNUC__) && __GNUC__ >= 15 && defined(__OPTIMIZE__)
   /* Ensure this is optimized as a tail call. */
   __attribute__((musttail)) return
 #endif
-  parse_key_values (argc, argv, false, magic_config_key);
+  parse_key_values (argc, argv, false, magic_config_key, cwd);
 }
 
-/* Parse a single [key=]value, calling config().
+/* Parse a single [key=]value or @PATH parameter, calling config().
  *
  * If the plugin provides magic_config_key then any "bare" values
  * (ones not containing "=") are prefixed with this key.
@@ -1183,32 +1200,114 @@ parse_key_values (int argc, char **argv, bool is_first,
  */
 static void
 parse_key_value (const char *arg,
-                 bool is_first, const char *magic_config_key)
+                 bool is_first, const char *magic_config_key, const char *cwd)
 {
-  const char *p, *key;
-  size_t n;
-
-  p = strchr (arg, '=');
-  n = p - arg;
-  if (p && is_config_key (arg, n)) { /* Is it key=value? */
-    key = nbdkit_strndup_intern (arg, n);
-    if (key == NULL)
-      exit (EXIT_FAILURE);
-    top->config (top, key, p+1);
+  if (arg[0] == '@' && strchr (arg, '/')) { /* @PATH */
+    parse_at_path (&arg[1], is_first, magic_config_key, cwd);
   }
-  else if (magic_config_key == NULL) {
-    if (is_first)               /* magic script parameter */
-      top->config (top, "script", arg);
-    else {
-      fprintf (stderr,
-               "%s: expecting key=value on the command line but got: %s\n",
-               program_name, arg);
+  else {                        /* [key=]value */
+    const char *p, *key;
+    size_t n;
+
+    p = strchr (arg, '=');
+    n = p - arg;
+    if (p && is_config_key (arg, n)) { /* Is it key=value? */
+      key = nbdkit_strndup_intern (arg, n);
+      if (key == NULL)
+        exit (EXIT_FAILURE);
+      top->config (top, key, p+1);
+    }
+    else if (magic_config_key == NULL) {
+      if (is_first)             /* magic script parameter */
+        top->config (top, "script", arg);
+      else {
+        fprintf (stderr,
+                 "%s: expecting key=value on the command line but got: %s\n",
+                 program_name, arg);
+        exit (EXIT_FAILURE);
+      }
+    }
+    else {                      /* magic config key */
+      top->config (top, magic_config_key, arg);
+    }
+  }
+}
+
+/* Parse @PATH parameter. */
+static void
+parse_at_path (const char *filename,
+               bool is_first, const char *magic_config_key, const char *cwd)
+{
+  FILE *fp;
+  const char *p;
+  CLEANUP_FREE char *path = NULL, *new_cwd = NULL, *line = NULL;
+  size_t linelen = 0, i;
+  ssize_t len;
+  const char *arg;
+
+  /* Calculate the path relative to the context directory. */
+  if (filename[0] == '/') {
+    path = strdup (filename);
+    if (path == NULL) {
+      fprintf (stderr, "%s: strdup: %m\n", program_name);
       exit (EXIT_FAILURE);
     }
   }
-  else {                        /* magic config key */
-    top->config (top, magic_config_key, arg);
+  else {
+    if (asprintf (&path, "%s/%s", cwd, filename) == -1) {
+      fprintf (stderr, "%s: asprintf: %m\n", program_name);
+      exit (EXIT_FAILURE);
+    }
   }
+
+  /* Calculate the new context directory in case this file contains
+   * any @PATH entries.  These are relative to this file.
+   */
+  p = strrchr (path, '/');
+  assert (p);
+  new_cwd = strndup (path, p-path);
+  if (new_cwd == NULL) {
+    fprintf (stderr, "%s: strndup: %m\n", program_name);
+    exit (EXIT_FAILURE);
+  }
+
+  /* Read the file line at a time, ignoring comments and blank lines. */
+  debug ("opening @%s", path);
+  fp = fopen (path, "r");
+  if (fp == NULL) {
+    fprintf (stderr, "%s: %s: %m\n", program_name, path);
+    exit (EXIT_FAILURE);
+  }
+
+  while ((len = getline (&line, &linelen, fp)) != -1) {
+    /* Ignore lines which are just whitespace or start with '#'. */
+    for (i = 0; i < len; ++i) {
+      if (line[i] == '#') {
+        i = len;
+        break;
+      }
+      if (!ascii_isspace (line[i]))
+        break;
+    }
+    if (i == len) continue;
+
+    /* Remove final \n if present. */
+    if (len > 0 && line[len-1] == '\n')
+      line[--len] = '\0';
+
+    /* We need to intern this string as it must live for the life of
+     * nbdkit.  This is freed at the end of main().
+     */
+    arg = nbdkit_strndup_intern (line, len);
+    if (arg == NULL)
+      exit (EXIT_FAILURE);
+
+    /* Parse it. */
+    parse_key_value (arg, is_first, magic_config_key, new_cwd);
+    is_first = false;
+  }
+
+  fclose (fp);
 }
 
 static void
